/**
 * Copyright (C) 2009-2010 Gauvain Pocentek <gpocentek@linutop.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif

#ifdef HAVE_SYS_WAIT_H
#include <sys/wait.h>
#endif

#ifdef HAVE_SYSLOG_H
#include <syslog.h>
#endif

#include <sys/utsname.h>

#include <gtk/gtk.h>

#include <libsoup/soup-cookie-jar-sqlite.h>

#include "main.h"
#include "utils.h"
#include "image-utils.h"
#include "video-utils.h"
#include "wk-browser.h"
#include "wk-window.h"
#include "wk-view.h"
#include "wk-logger.h"

enum
{
    VIEW_PAGE_WEB = 0,
    VIEW_PAGE_IMAGE,
    VIEW_PAGE_VIDEO,
    VIEW_PAGE_BLACK
};

/* signals */
enum
{
    SESSION_START_SIGNAL = 0,
    SESSION_STOP_SIGNAL,
    TIME_ADDED_SIGNAL,
    SLIDESHOW_START_SIGNAL,
    LAST_SIGNAL
};

static guint fe_signals[LAST_SIGNAL] = { 0 };

gboolean can_continue = TRUE;
guint context_id;

static void _view_build_toolbar   (WkView *view);
static void _view_build_statusbar (WkView *view);

static void _view_on_webkit_load_status (WebKitWebView *webkitwebview,
                                         GParamSpec    *pspec,
                                         gpointer       data);
static void _view_notify_progress       (WebKitWebView *webkitwebview,
                                         GParamSpec    *pspec,
                                         gpointer       data);
static void _view_link_hover            (WebKitWebView *webkitwebview,
                                         const gchar   *title,
                                         const gchar   *link,
                                         gpointer       data);

static gboolean _view_check_policy (WebKitWebView             *web_view,
                                    WebKitWebFrame            *frame,
                                    WebKitNetworkRequest      *request,
                                    WebKitWebNavigationAction *navigation_action,
                                    WebKitWebPolicyDecision   *policy_decision,
                                    gpointer                   user_data);
static WebKitWebView *_view_create_web_view (WebKitWebView  *web_view,
                                             WebKitWebFrame *frame,
                                             gpointer        data);
static gboolean _view_check_mime_policy (WebKitWebView           *web_view,
                                         WebKitWebFrame          *frame,
                                         WebKitNetworkRequest    *request,
                                         gchar                   *mimetype,
                                         WebKitWebPolicyDecision *policy_decision,
                                         gpointer                 user_data);
static gboolean _view_do_download (WebKitWebView *web_view,
                                   GObject       *download,
                                   gpointer       user_data);
static void _view_download_notify_status (WebKitDownload *download,
                                          GParamSpec     *pspec,
                                          gpointer        user_data);

static void _view_go                  (GtkWidget *widget, gpointer data);
static void _view_go_home             (GtkWidget *widget, gpointer data);
static void _view_stop_loading        (GtkWidget *widget, gpointer data);
static void _view_go_back             (GtkWidget *widget, gpointer view);
static void _view_go_forward          (GtkWidget *widget, gpointer view);
static void _view_set_current_view    (GtkWidget *entry, gpointer data);
static gboolean _view_url_entry_modified (GtkWidget   *entry,
                                          GdkEventKey *event,
                                          gpointer     data);
static void _view_set_history_buttons (WkView *view);

static void _view_ask_stop_session (GtkWidget *widget, gpointer data);
static void _view_clear_session    (GtkWidget *widget, gpointer data);

static gboolean _view_post_session_uri (WkView *view);

static gboolean _view_decrease_time (WkView *view);

static void     _view_kill_slideshow_timeout (WkView *view);
static gboolean _view_show_current_item      (WkView  *view,
                                              gboolean set_timeout);
static void     _view_display_set_timeout    (WkView        *view,
                                              SlideshowItem *item,
                                              gboolean       next);
static void     _view_on_video_end           (WkVideo *video,
                                              gpointer data);

static gboolean _view_show_next_item     (WkView *view);
static gboolean _view_show_next_dir_item (WkView *view);

static void on_auth_requested (SoupSession *session,
                               SoupMessage *msg,
                               SoupAuth *auth,
                               gboolean retrying,
                               gpointer data);

struct _WkViewPrivate
{
    /* toolbar */
    GtkWidget   *toolbar;
    GtkWidget   *urlEntry;
    GtkWidget   *timeLabel;
    GtkToolItem *backwardBtn;
    GtkToolItem *forwardBtn;
    GtkToolItem *homeBtn;
    GtkToolItem *stopBtn;
    GtkToolItem *clearBtn;
    GtkToolItem *resetBtn;

    /* statusbar */
    GtkWidget *statusbar;
    GtkWidget *progressbar;

    /* timer */
    guint time;
    guint time_timeout;

    /* slideshow */
    guint  slideshow_timeout;
    GList *slideshow_current;
    GList *playlist;

    /* browser */
    WkBrowser  *browser;
    WkSettings *settings;

    /* Soup */
    SoupSession *session;

    /* Misc */
    WkViewState state;
    gboolean    entry_edited;
    GtkWidget  *black_img;
};

static void wk_view_class_init (WkViewClass *klass);
static void wk_view_init (WkView *view);
static void wk_view_finalize (GObject *object);

G_DEFINE_TYPE (WkView, wk_view, GTK_TYPE_NOTEBOOK);

static void
wk_view_class_init (WkViewClass *klass)
{
    g_type_class_add_private (klass, sizeof (WkViewPrivate));

    GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
    gobject_class->finalize = wk_view_finalize;

    fe_signals[SESSION_START_SIGNAL] = g_signal_new ("session-start",
        G_TYPE_FROM_CLASS (klass),
        G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
        G_STRUCT_OFFSET (WkViewClass, session_start),
        NULL,
        NULL,
        g_cclosure_marshal_VOID__VOID,
        G_TYPE_NONE,
        0);

    fe_signals[SESSION_STOP_SIGNAL] = g_signal_new ("session-stop",
        G_TYPE_FROM_CLASS (klass),
        G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
        G_STRUCT_OFFSET (WkViewClass, session_end),
        NULL,
        NULL,
        g_cclosure_marshal_VOID__VOID,
        G_TYPE_NONE,
        0);

    fe_signals[TIME_ADDED_SIGNAL] = g_signal_new ("time-added",
        G_TYPE_FROM_CLASS (klass),
        G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
        G_STRUCT_OFFSET (WkViewClass, time_added),
        NULL,
        NULL,
        g_cclosure_marshal_VOID__VOID,
        G_TYPE_NONE,
        0);

    fe_signals[SLIDESHOW_START_SIGNAL] = g_signal_new ("slideshow-start",
        G_TYPE_FROM_CLASS (klass),
        G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
        G_STRUCT_OFFSET (WkViewClass, slideshow_start),
        NULL,
        NULL,
        g_cclosure_marshal_VOID__VOID,
        G_TYPE_NONE,
        0);
}

static void
wk_view_init (WkView *view)
{
    const gchar *proxy_uri;
    GtkWidget   *sc, *vbox;
    gchar *user_agent = NULL;
    gchar *sys_name = NULL;
    struct utsname name;
    WebKitWebSettings *web_settings;
    SoupURI     *soup_proxy_uri;

    vbox = gtk_vbox_new (FALSE, 0);

    view->priv = G_TYPE_INSTANCE_GET_PRIVATE (view,
                                              WK_TYPE_VIEW,
                                              WkViewPrivate);

    view->priv->browser = WK_BROWSER (wk_browser_get (NULL));
    view->priv->settings = wk_browser_get_settings (view->priv->browser);

    view->priv->playlist = wk_settings_expand_playlist (view->priv->settings);
    view->priv->slideshow_current = NULL;
    view->priv->slideshow_timeout = 0;

    view->priv->time = 0;
    view->priv->time_timeout = 0;

    view->priv->state = WK_VIEW_STATE_DISPLAY;
    view->priv->entry_edited = false;

    _view_build_toolbar (view);
    _view_build_statusbar (view);

    gtk_box_pack_start (GTK_BOX (vbox), view->priv->toolbar, FALSE, FALSE, 0);

    sc = gtk_scrolled_window_new (NULL, NULL);
    gtk_box_pack_start (GTK_BOX (vbox), sc, TRUE, TRUE, 0);
    gtk_widget_show (sc);
    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sc),
                                    GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

    view->webview = WEBKIT_WEB_VIEW (webkit_web_view_new ());
    g_signal_connect (G_OBJECT (view->webview), "notify::load-status",
                      G_CALLBACK (_view_on_webkit_load_status), view);
    g_signal_connect (G_OBJECT (view->webview), "notify::progress",
                      G_CALLBACK (_view_notify_progress), view);
    g_signal_connect (G_OBJECT (view->webview), "hovering-over-link",
                      G_CALLBACK (_view_link_hover), view);
    g_signal_connect (view->webview, "navigation-policy-decision-requested",
                      G_CALLBACK (_view_check_policy), view);
    g_signal_connect (view->webview, "mime-type-policy-decision-requested",
                      G_CALLBACK (_view_check_mime_policy), view);
    g_signal_connect (view->webview, "create-web-view",
                      G_CALLBACK (_view_create_web_view), view);
    g_signal_connect (view->webview, "download-requested",
                      G_CALLBACK (_view_do_download), view);
    gtk_container_add (GTK_CONTAINER (sc), GTK_WIDGET (view->webview));

    gtk_box_pack_start (GTK_BOX (vbox), view->priv->statusbar, FALSE, FALSE, 0);

    view->image = gtk_image_new ();
    view->video = wk_video_new (view->priv->settings->slideshow_mute);
    g_signal_connect (G_OBJECT (view->video), "end-reached",
                      G_CALLBACK (_view_on_video_end), view);

    view->priv->session = webkit_get_default_session ();

    /* define the user agent, borrowed from midori */
    if (uname (&name) != -1)
        sys_name = g_strdup(name.sysname);
    else
        sys_name = g_strdup ("Unix");
    user_agent = g_strdup_printf ("%s/%s (%s; %s; U; %s) WebKit/531.2+",
                   "LinutopKiosk",
                   PACKAGE_VERSION,
                   "X11", sys_name,
                   pango_language_to_string (gtk_get_default_language ()));
    g_object_set (view->priv->session, "user-agent", user_agent, NULL);

    web_settings = webkit_web_view_get_settings (view->webview);
    g_object_set (web_settings,
                    "WebKitWebSettings::enable-scripts", TRUE,
                    "WebKitWebSettings::user-agent", user_agent,
                     NULL);

    g_free (sys_name);
    g_free (user_agent);

    /* set the proxy URI if any */
    if ((proxy_uri = g_getenv ("http_proxy")) != NULL)
    {
        soup_proxy_uri = soup_uri_new (proxy_uri);
        g_object_set (view->priv->session, "proxy-uri", soup_proxy_uri, NULL);
        soup_uri_free (soup_proxy_uri);
    }

    /* notebook properties */
    g_object_set (G_OBJECT (view), "show-border", FALSE,
                                   "show-tabs", FALSE,
                                   NULL);

    view->priv->black_img = gtk_image_new ();

    gtk_notebook_append_page (GTK_NOTEBOOK (view), vbox, NULL);
    gtk_notebook_append_page (GTK_NOTEBOOK (view), view->image, NULL);
    gtk_notebook_append_page (GTK_NOTEBOOK (view), view->video, NULL);
    gtk_notebook_append_page (GTK_NOTEBOOK (view), view->priv->black_img, NULL);
}

static void
wk_view_finalize (GObject *object)
{
    WkView *view = WK_VIEW (object);
    GSource *source;

    if (view->priv->time_timeout)
    {
        if ((source = g_main_context_find_source_by_id (
                          NULL, view->priv->time_timeout)))
            g_source_destroy (source);
    }

    _view_kill_slideshow_timeout (view);
}



/*
 * Internal callbacks
 */
static void
_view_on_webkit_load_status (WebKitWebView *webkitwebview,
                             GParamSpec    *spec,
                             gpointer       data)
{
    const gchar *uri;
    WkView *view;
    WebKitWebFrame *frame;

    view = WK_VIEW (data);

    switch (webkit_web_view_get_load_status (view->webview))
    {
    case WEBKIT_LOAD_COMMITTED:
        gtk_widget_set_sensitive (GTK_WIDGET (view->priv->stopBtn), TRUE);
        frame = webkit_web_view_get_main_frame (view->webview);
        uri = webkit_web_frame_get_uri (frame);
        if (uri)
            gtk_entry_set_text (GTK_ENTRY (view->priv->urlEntry), uri);
        break;

    case WEBKIT_LOAD_PROVISIONAL:
    case WEBKIT_LOAD_FIRST_VISUALLY_NON_EMPTY_LAYOUT:
        gtk_widget_set_sensitive (GTK_WIDGET (view->priv->stopBtn), TRUE);
        break;

    case WEBKIT_LOAD_FINISHED:
        gtk_widget_set_sensitive (GTK_WIDGET (view->priv->stopBtn), FALSE);
        uri = webkit_web_view_get_uri (webkitwebview);
        if (uri && ! view->priv->entry_edited)
        {
            gtk_entry_set_text (GTK_ENTRY (view->priv->urlEntry), uri);
            view->priv->entry_edited = false;
        }
        gtk_notebook_set_current_page (GTK_NOTEBOOK (view), VIEW_PAGE_WEB);
        break;

    default:
        gtk_widget_set_sensitive (GTK_WIDGET (view->priv->stopBtn), FALSE);
        break;
    }
    _view_set_history_buttons (view);
}

static void
_view_notify_progress (WebKitWebView *webkitwebview,
                       GParamSpec    *pspec,
                       gpointer       data)
{
    WkView *view;
    gdouble progress;

    view = WK_VIEW (data);
    progress = webkit_web_view_get_progress (view->webview);

    if (progress == 1.0)
        gtk_widget_hide (view->priv->progressbar);
    else
    {
        gtk_widget_show (view->priv->progressbar);
        gtk_progress_bar_set_fraction (
            GTK_PROGRESS_BAR (view->priv->progressbar), progress);
    }
}

static void
_view_link_hover (WebKitWebView *webkitwebview,
                  const gchar   *title,
                  const gchar   *link,
                  gpointer       data)
{
    WkView *view;

    view = WK_VIEW (data);

    gtk_statusbar_pop (GTK_STATUSBAR (view->priv->statusbar), context_id);
    if (link)
        gtk_statusbar_push (GTK_STATUSBAR (view->priv->statusbar),
                            context_id, link);
}

static void
_view_set_history_buttons (WkView *view)
{
    gtk_widget_set_sensitive (GTK_WIDGET (view->priv->backwardBtn),
        webkit_web_view_can_go_back (view->webview));
    gtk_widget_set_sensitive (GTK_WIDGET (view->priv->forwardBtn),
        webkit_web_view_can_go_back_or_forward (view->webview, 1));
}

static void
_view_go (GtkWidget *widget, gpointer data)
{
    gtk_widget_activate (WK_VIEW (data)->priv->urlEntry);
}

static void
_view_stop_loading (GtkWidget *widget, gpointer data)
{
    WkView *view = WK_VIEW (data);

    webkit_web_view_stop_loading (view->webview);
    gtk_widget_set_sensitive (GTK_WIDGET (view->priv->stopBtn), FALSE);
}

static void
_view_go_home (GtkWidget *widget, gpointer view)
{
    webkit_web_view_load_uri (WK_VIEW (view)->webview,
                              WK_VIEW (view)->priv->settings->default_url);
}

static void
_view_go_back (GtkWidget *widget, gpointer view)
{
    webkit_web_view_go_back (WK_VIEW (view)->webview);
    _view_set_history_buttons (WK_VIEW (view));
}

static void
_view_go_forward (GtkWidget *widget, gpointer view)
{
    webkit_web_view_go_forward (WK_VIEW (view)->webview);
    _view_set_history_buttons (WK_VIEW (view));
}

static void
_view_ask_stop_session (GtkWidget *widget, gpointer data)
{
    GtkWidget *dialog;
    gint response;

    WkView *view = WK_VIEW (data);

    dialog = gtk_message_dialog_new (NULL,
                            GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
                            GTK_MESSAGE_QUESTION,
                            GTK_BUTTONS_YES_NO,
                            _("Would you really like to end this session?"));
    response = gtk_dialog_run (GTK_DIALOG (dialog));

    gtk_widget_destroy (dialog);

    if (response == GTK_RESPONSE_YES)
        wk_view_stop_session (view);
}

static void
_view_clear_session (GtkWidget *widget, gpointer data)
{
    LOG ("%s", "CLEAR (USER)");
    wk_browser_clear_session (WK_VIEW (data)->priv->browser, FALSE);
}

static gboolean
_view_url_entry_modified (GtkWidget *entry, GdkEventKey *event, gpointer data)
{
    WkView *view;

    view = WK_VIEW (data);
    view->priv->entry_edited = true;

    return FALSE;
}

static void
_view_set_current_view (GtkWidget *entry, gpointer data)
{
    gchar *uri = NULL;
    WkView *view;
    WkSettings *settings;

    uri = g_strdup (gtk_entry_get_text (GTK_ENTRY (entry)));
    settings = GET_SETTINGS();
    view = WK_VIEW (data);

    if (G_UNLIKELY (!uri))
        return;

    view->priv->entry_edited = false;

    /* parse the uri for specific keywords */
    if ((g_strcmp0 (uri, "exit") == 0)
        || (g_strcmp0 (uri, "quit") == 0)
        || (g_strcmp0 (uri, "bye") == 0))
    {
        wk_browser_maybe_quit (view->priv->browser);
    }

    expand_uri (&uri);

    _view_set_history_buttons (view);
    webkit_web_view_load_uri (view->webview, uri);

    g_free (uri);
}

static void
_view_check_in_blacklist (gpointer data, gpointer user_data)
{
    const gchar *uri;
    const gchar *regex;

    uri = (const gchar *) user_data;
    regex = (const gchar *) data;

    if (g_regex_match_simple (regex, uri, G_REGEX_CASELESS, 0))
        can_continue = FALSE;
}

static void
_view_check_in_whitelist (gpointer data, gpointer user_data)
{
    const gchar *uri;
    const gchar *regex;

    uri = (const gchar *) user_data;
    regex = (const gchar *) data;

    printf ("uri: %s\nregex: %s\n", uri, regex);

    if (g_regex_match_simple (regex, uri, G_REGEX_CASELESS, 0))
        can_continue = TRUE;
}

static gboolean
_view_check_policy (WebKitWebView             *web_view,
                    WebKitWebFrame            *frame,
                    WebKitNetworkRequest      *request,
                    WebKitWebNavigationAction *navigation_action,
                    WebKitWebPolicyDecision   *policy_decision,
                    gpointer                   data)
{
    WkView *view;
    const gchar *uri;
    WkSettings *settings;

    view = (WkView *) data;
    settings = GET_SETTINGS();
    uri = webkit_network_request_get_uri (request);

    if (view->priv->slideshow_timeout)
    {
        webkit_web_policy_decision_use (policy_decision);
        return TRUE;
    }

    if (g_str_has_prefix (uri, "file://")
        && ! settings->can_access_fs)
    {
        webkit_web_policy_decision_ignore (policy_decision);

        LOG ("URL (denied): %s", uri);
        gtk_statusbar_pop (GTK_STATUSBAR (view->priv->statusbar), context_id);
        gtk_statusbar_push (GTK_STATUSBAR (view->priv->statusbar),
                            context_id, _("Access denied."));

        return TRUE;
    }

    if (settings->list_mode == LIST_MODE_BLACKLIST)
    {
        can_continue = TRUE;
        g_list_foreach (settings->list, _view_check_in_blacklist, (gpointer) uri);
    }

    if (settings->list_mode == LIST_MODE_WHITELIST)
    {
        can_continue = FALSE;
        g_list_foreach (settings->list, _view_check_in_whitelist, (gpointer) uri);
    }

    if (! can_continue)
    {
        webkit_web_policy_decision_ignore (policy_decision);
        LOG ("URL (denied): %s", uri);

        gtk_statusbar_pop (GTK_STATUSBAR (view->priv->statusbar), context_id);
        gtk_statusbar_push (GTK_STATUSBAR (view->priv->statusbar),
                            context_id, _("Access denied."));

        return TRUE;
    }

    LOG ("URL (allowed): %s", uri);

    webkit_web_policy_decision_use (policy_decision);

    return TRUE;
}

static gboolean
_view_web_view_ready (WebKitWebView *web_view, gpointer data)
{
    gtk_widget_show_all (GTK_WIDGET (data));

    return FALSE;
}

static WebKitWebView *
_view_create_web_view (WebKitWebView  *web_view,
                       WebKitWebFrame *frame,
                       gpointer        data)
{
    GtkWidget *view;
    GtkWidget *window;
    WkSettings *settings;
    WkBrowser *browser;

    browser = WK_BROWSER (wk_browser_get (NULL));
    window = wk_window_new (WK_WINDOW_TYPE_CHILD, NULL);
    settings = GET_SETTINGS();

    if ((settings->max_windows > 0)
        && (settings->max_windows <= wk_browser_get_windows_count (browser)))
    {
        return NULL;
    }

    /* notify the browser that a new window has been opened */
    wk_browser_register_window (browser, WK_WINDOW (window));

    view = wk_window_get_view (WK_WINDOW (window));

    g_signal_connect (G_OBJECT (WK_VIEW(view)->webview), "web-view-ready",
                      G_CALLBACK (_view_web_view_ready), window);

    return WK_VIEW(view)->webview;
}


static gboolean
_view_check_mime_policy (WebKitWebView           *web_view,
                            WebKitWebFrame          *frame,
                            WebKitNetworkRequest    *request,
                            gchar                   *mimetype,
                            WebKitWebPolicyDecision *policy_decision,
                            gpointer                 user_data)
{
    WkView *view;

    view = WK_VIEW (user_data);

    if (view->priv->settings->mime_associations
        && g_hash_table_lookup (view->priv->settings->mime_associations,
                                mimetype) != NULL)
    {
        webkit_web_policy_decision_download (policy_decision);
        return TRUE;
    }

    return FALSE;
}

static void
_view_download_notify_status (WebKitDownload *download,
                                 GParamSpec     *pspec,
                                 gpointer        user_data)
{
    if (webkit_download_get_status (download) != WEBKIT_DOWNLOAD_STATUS_FINISHED)
        return;

    gchar *handler = NULL;
    gchar *filename = NULL;
    WebKitNetworkResponse *response;
    SoupMessage *message;
    const gchar *mimetype;
    gchar *cmd;
    int argc;
    gchar **argv;
    GPid pid;
    WkBrowser *browser;

    browser = WK_BROWSER (wk_browser_get (NULL));

    WkView *view = WK_VIEW (user_data);
    response = webkit_download_get_network_response (download);
    message = webkit_network_response_get_message (response);
    mimetype = soup_message_headers_get_content_type (message->response_headers, NULL);

    if (!view->priv->settings->mime_associations)
        return;

    handler = g_hash_table_lookup (view->priv->settings->mime_associations, mimetype);
    filename = g_filename_from_uri (webkit_download_get_destination_uri (download),
                                    NULL, NULL);

    cmd = g_strdup_printf ("%s '%s'", handler, filename);
    if (g_shell_parse_argv (cmd, &argc, &argv, NULL))
    {
        if (g_spawn_async (NULL, argv, NULL, G_SPAWN_SEARCH_PATH,
                           NULL, NULL, &pid, NULL))
        {
            wk_browser_register_external (browser, pid);
        }
    }

    g_free (filename);
    g_strfreev (argv);
    g_free (cmd);
}

static gboolean
_view_do_download (WebKitWebView *web_view,
                      GObject       *download_obj,
                      gpointer       user_data)
{
    gchar *filename = NULL;
    gchar *uri = NULL;

    WebKitDownload *download = (WebKitDownload *) download_obj;
    WkView *view = WK_VIEW (user_data);


    filename = g_build_filename (g_get_tmp_dir (),
                                 view->priv->settings->tmpdir_name,
                                 webkit_download_get_suggested_filename (download),
                                 NULL);
    uri = g_filename_to_uri (filename, NULL, NULL);

    webkit_download_set_destination_uri (download, uri);
    g_signal_connect (download, "notify::status",
        G_CALLBACK (_view_download_notify_status), user_data);

    LOG ("DOWNLOAD: %s", uri);

    g_free (uri);
    g_free (filename);

    return TRUE;
}

static void
_view_build_statusbar (WkView *view)
{
    view->priv->statusbar = gtk_statusbar_new ();
    view->priv->progressbar = gtk_progress_bar_new ();
    gtk_box_pack_end (GTK_BOX (view->priv->statusbar), view->priv->progressbar,
                      FALSE, FALSE, 0);

    context_id = gtk_statusbar_get_context_id (
                    GTK_STATUSBAR (view->priv->statusbar), "Link Hover");
}

static void
_view_build_toolbar (WkView *view)
{
    GtkToolItem *item;

    view->priv->toolbar = gtk_toolbar_new ();
    gtk_toolbar_set_style (GTK_TOOLBAR (view->priv->toolbar),
                           GTK_TOOLBAR_ICONS);

    view->priv->backwardBtn = gtk_tool_button_new_from_stock (GTK_STOCK_GO_BACK);
    gtk_widget_set_sensitive (GTK_WIDGET (view->priv->backwardBtn), FALSE);
    gtk_widget_set_tooltip_text (GTK_WIDGET (view->priv->backwardBtn),
                                 _("Go back"));
    g_signal_connect (G_OBJECT (view->priv->backwardBtn), "clicked",
                      G_CALLBACK (_view_go_back), view);
    gtk_toolbar_insert (GTK_TOOLBAR (view->priv->toolbar),
                        view->priv->backwardBtn,
                        -1);

    view->priv->forwardBtn = gtk_tool_button_new_from_stock (GTK_STOCK_GO_FORWARD);
    gtk_widget_set_sensitive (GTK_WIDGET (view->priv->forwardBtn), FALSE);
    gtk_widget_set_tooltip_text (GTK_WIDGET (view->priv->forwardBtn),
                                 _("Go forward"));
    g_signal_connect (G_OBJECT (view->priv->forwardBtn), "clicked",
                      G_CALLBACK (_view_go_forward), view);
    gtk_toolbar_insert (GTK_TOOLBAR (view->priv->toolbar),
                        view->priv->forwardBtn,
                        -1);

    view->priv->homeBtn = gtk_tool_button_new_from_stock (GTK_STOCK_HOME);
    gtk_widget_set_tooltip_text (GTK_WIDGET (view->priv->forwardBtn),
                                 _("Go to home page"));
    g_signal_connect (G_OBJECT (view->priv->homeBtn), "clicked",
                      G_CALLBACK (_view_go_home), view);
    gtk_toolbar_insert (GTK_TOOLBAR (view->priv->toolbar),
                        view->priv->homeBtn,
                        -1);

    view->priv->stopBtn = gtk_tool_button_new_from_stock (GTK_STOCK_STOP);
    gtk_widget_set_tooltip_text (GTK_WIDGET (view->priv->forwardBtn),
                                 _("Stop loading"));
    gtk_widget_set_sensitive (GTK_WIDGET (view->priv->stopBtn), FALSE);
    g_signal_connect (G_OBJECT (view->priv->stopBtn), "clicked",
                      G_CALLBACK (_view_stop_loading), view);
    gtk_toolbar_insert (GTK_TOOLBAR (view->priv->toolbar),
                        view->priv->stopBtn,
                        -1);

    item = gtk_tool_item_new ();
    gtk_tool_item_set_expand (item, TRUE);
    view->priv->urlEntry = gtk_entry_new ();
    gtk_container_add (GTK_CONTAINER (item), view->priv->urlEntry);
    g_signal_connect (G_OBJECT (view->priv->urlEntry), "activate",
                      G_CALLBACK (_view_set_current_view), view);
    g_signal_connect (G_OBJECT (view->priv->urlEntry), "key-press-event",
                      G_CALLBACK (_view_url_entry_modified), view);
    gtk_toolbar_insert (GTK_TOOLBAR (view->priv->toolbar), item, -1);

    item = gtk_tool_button_new_from_stock (GTK_STOCK_OK);
    gtk_widget_set_tooltip_text (GTK_WIDGET (item),
                                 _("Go to the address"));
    g_signal_connect (G_OBJECT (item), "clicked",
                      G_CALLBACK (_view_go), view);
    gtk_toolbar_insert (GTK_TOOLBAR (view->priv->toolbar), item, -1);

    if (view->priv->settings->with_timer)
    {
        item = gtk_tool_item_new ();
        view->priv->timeLabel = gtk_label_new ("00:00");
        gtk_widget_set_tooltip_text (GTK_WIDGET (view->priv->timeLabel),
                                     _("Remaining session time"));
        gtk_container_add (GTK_CONTAINER (item), view->priv->timeLabel);
        gtk_toolbar_insert (GTK_TOOLBAR (view->priv->toolbar), item, -1);
    }

    if (view->priv->settings->with_clear)
    {
        view->priv->clearBtn = gtk_tool_button_new_from_stock (GTK_STOCK_CLEAR);
        gtk_widget_set_tooltip_text (GTK_WIDGET (view->priv->clearBtn),
                                     _("Clear history and cookies"));
        g_signal_connect (G_OBJECT (view->priv->clearBtn), "clicked",
                          G_CALLBACK (_view_clear_session), view);
        gtk_toolbar_insert (GTK_TOOLBAR (view->priv->toolbar),
                            view->priv->clearBtn,
                            -1);
    }

    if (view->priv->settings->can_end_session)
    {
        view->priv->resetBtn = gtk_tool_button_new_from_stock (GTK_STOCK_QUIT);
        gtk_widget_set_tooltip_text (GTK_WIDGET (view->priv->resetBtn),
                                     _("End this session"));
        g_signal_connect (G_OBJECT (view->priv->resetBtn), "clicked",
                          G_CALLBACK (_view_ask_stop_session), view);
        gtk_toolbar_insert (GTK_TOOLBAR (view->priv->toolbar),
                            view->priv->resetBtn,
                            -1);
    }
}

static void
del_slide_item (gpointer data, gpointer user_data)
{
    SlideshowItem *item;

    item = (SlideshowItem *) data;

    g_free ((gchar *)item->uri);
    g_free (item);
}

static void
_view_set_image (const gchar *uri, WkView *view)
{
    const gchar *filename;
    GdkPixbuf *pxb;

    if (g_str_has_prefix (uri, "file://"))
        filename = uri+7;
    else
        filename = uri;

    pxb = get_pixbuf_from_filename (filename, view);
    gtk_image_set_from_pixbuf (GTK_IMAGE (view->image), pxb);
    gtk_notebook_set_current_page (GTK_NOTEBOOK (view), VIEW_PAGE_IMAGE);

    g_object_unref (pxb);
}

gulong signal_id;

static void
on_auth_requested (SoupSession *session, SoupMessage *msg, SoupAuth *auth,
                   gboolean retrying, gpointer data)
{
    const gchar *url;
    SoupURI *uri;

    url = (const gchar *) data;

    g_signal_handler_disconnect (session, signal_id);

    uri = soup_uri_new (url);
    soup_auth_authenticate (auth, uri->user, uri->password);
}

static void
_view_set_webpage (const gchar *uri, WkView *view)
{
    signal_id = g_signal_connect (G_OBJECT (view->priv->session), "authenticate",
                      G_CALLBACK (on_auth_requested), (gpointer) uri);

    webkit_web_view_load_uri (view->webview, uri);
}

static gboolean
_view_after_black (WkView *view)
{
    wk_view_select_item_next (view, FALSE, FALSE);
    _view_show_current_item (view, TRUE);

    return FALSE;
}

static void
_view_on_video_end (WkVideo *video, gpointer data)
{
    WkView *view;

    view = WK_VIEW (data);

    gtk_image_set_black (view->priv->black_img, view);

    g_timeout_add (100, (GSourceFunc) _view_after_black, view);
}

static void
_view_set_video (const gchar *uri, WkView *view)
{
    _view_kill_slideshow_timeout (view);

    gtk_notebook_set_current_page (GTK_NOTEBOOK (view), VIEW_PAGE_VIDEO);
    wk_video_play (WK_VIDEO (view->video), uri);
}

static void
_view_display_set_timeout (WkView *view, SlideshowItem *item, gboolean next)
{
    _view_kill_slideshow_timeout (view);

    if (next)
        view->priv->slideshow_timeout = g_timeout_add_seconds (
            item->time, (GSourceFunc) _view_show_next_item, view);
    else
        view->priv->slideshow_timeout = g_timeout_add_seconds (
            item->time, (GSourceFunc) _view_show_next_dir_item, view);
}

static void
_view_kill_slideshow_timeout (WkView *view)
{
    GSource *source;

    if (!view->priv->slideshow_timeout)
        return;

    source = g_main_context_find_source_by_id (
                NULL, view->priv->slideshow_timeout);
    if (source)
        g_source_destroy (source);
    view->priv->slideshow_timeout = 0;
}

static gboolean
_view_show_next_item (WkView *view)
{
    wk_view_select_item_next (view, FALSE, FALSE);
    _view_show_current_item (view, TRUE);

    return FALSE;
}

static gboolean
_view_show_next_dir_item (WkView *view)
{
    _view_show_current_item (view, TRUE);

    return FALSE;
}

static gboolean
_view_show_current_item (WkView *view, gboolean set_timeout)
{
    SlideshowItem *item = NULL;

    if (!view->priv->playlist)
        return FALSE;

    item = (SlideshowItem *) view->priv->slideshow_current->data;
    webkit_web_view_stop_loading (view->webview);

    if (is_video_uri (item->uri))
    {
        _view_set_video (item->uri, view);
        return FALSE;
    }
    else if (is_local_image_uri (item->uri))
    {
        _view_set_image (item->uri, view);
        if (set_timeout)
            _view_display_set_timeout (view, item, TRUE);
        return TRUE;
    }
    else
    {
        _view_set_webpage (item->uri, view);
        if (set_timeout)
            _view_display_set_timeout (view, item, TRUE);
        return TRUE;
    }
}

static gboolean
_view_post_session_uri (WkView *view)
{
    WkSettings *settings;
    settings = GET_SETTINGS();

    printf ("%d\n", settings->after_web_session);
    if (settings->after_web_session == AFTER_WEB_SESSION_SLIDESHOW)
        wk_view_start_slideshow (view);
    else if (settings->after_web_session == AFTER_WEB_SESSION_SESSION)
        wk_view_start_session (view);
    else
        _view_go_home (NULL, view);

    return FALSE;
}

static gboolean
_view_decrease_time (WkView *view)
{
    if (G_LIKELY (--(view->priv->time) > 0)) {
        gchar *time_str = NULL;
        seconds_to_string (view->priv->time, &time_str);
        gtk_label_set_text (GTK_LABEL (view->priv->timeLabel), time_str);
        g_free (time_str);
        return TRUE;
    }

    wk_view_start_slideshow (view);

    return FALSE;
}

/*
 * Public API
 */
GtkWidget *
wk_view_new ()
{
    WkView *view = g_object_new (WK_TYPE_VIEW, NULL);
    return GTK_WIDGET (view);
}

WkViewState
wk_view_get_state (WkView *view)
{
    return view->priv->state;
}

void
wk_view_set_url (WkView *view, gchar *url)
{
    webkit_web_view_load_uri (view->webview, url);
}

void
wk_view_show_toolbar (WkView *view, gboolean show)
{
    if (show && ! view->priv->settings->hide_toolbar)
    {
        gtk_widget_show (view->priv->toolbar);
        gtk_widget_show (view->priv->statusbar);
    }
    else
    {
        gtk_widget_hide (view->priv->toolbar);
        gtk_widget_hide (view->priv->statusbar);
    }
}

void
wk_view_start_slideshow (WkView *view)
{
    SoupCookieJar *jar;

    g_signal_emit (G_OBJECT (view), fe_signals[SLIDESHOW_START_SIGNAL], 0);

    view->priv->state = WK_VIEW_STATE_DISPLAY;

    gdk_event_handler_set (event_filter, NULL, NULL);
    if (view->priv->settings->slideshow_hide_pointer)
        start_unclutter ();

    wk_view_show_toolbar (view, FALSE);

    /* remove the webkit auth dialog if it exists */
    soup_session_remove_feature_by_type (view->priv->session,
                                         g_type_from_name ("WebKitSoupAuthDialog"));

    /* read the cookies */
    if (view->priv->settings->cookies_db)
    {
        jar = soup_cookie_jar_sqlite_new (view->priv->settings->cookies_db,
                                          TRUE);
        soup_session_add_feature (view->priv->session,
                                  SOUP_SESSION_FEATURE (jar));
    }

    if (!view->priv->playlist)
    {
        webkit_web_view_load_uri (view->webview,
                                  view->priv->settings->default_url);
        return;
    }

    view->priv->slideshow_current = NULL;
    wk_view_select_item_next (view, FALSE, FALSE);
    _view_show_current_item (view, TRUE);
}

void
wk_view_stop_slideshow (WkView *view)
{
    if (!view->priv->slideshow_timeout)
        return;

    soup_session_add_feature_by_type (view->priv->session,
                                      g_type_from_name ("WebKitSoupAuthDialog"));
    soup_session_remove_feature_by_type (view->priv->session,
                                         soup_cookie_jar_get_type ());

    _view_kill_slideshow_timeout (view);

    view->priv->state = WK_VIEW_STATE_NONE;
}

guint
wk_view_get_time (WkView *view)
{
    return view->priv->time;
}

void
wk_view_start_session (WkView *view)
{
    WebKitWebBackForwardList *list;

    LOG ("%s", "START");

    stop_unclutter ();
    gdk_event_handler_set (event_no_filter, NULL, NULL);

    if (view->priv->state == WK_VIEW_STATE_WEB)
        return;

    wk_view_stop_slideshow (view);
    wk_view_show_toolbar (view, TRUE);

    view->priv->state = WK_VIEW_STATE_WEB;
    g_signal_emit (G_OBJECT (view), fe_signals[SESSION_START_SIGNAL], 0);

    list = webkit_web_back_forward_list_new_with_web_view (view->webview);
    webkit_web_back_forward_list_set_limit (list,
                                            view->priv->settings->max_history);

    soup_session_add_feature_by_type (view->priv->session,
                                      g_type_from_name ("WebKitSoupAuthDialog"));

    soup_session_add_feature_by_type (view->priv->session,
                                      soup_cookie_jar_get_type ());

    webkit_web_view_load_uri (view->webview, view->priv->settings->default_url);
    _view_set_history_buttons (view);

    if (view->priv->settings->with_timer)
    {
        view->priv->time_timeout = g_timeout_add_seconds (
                                    1, (GSourceFunc) _view_decrease_time,
                                    view);
    }
}

void
wk_view_stop_session (WkView *view)
{
    WebKitWebBackForwardList *list;

    if (view->priv->state != WK_VIEW_STATE_WEB
        && view->priv->state != WK_VIEW_STATE_NONE)
            return;

    LOG ("%s", "STOP");

    view->priv->time = 0;
    if (view->priv->time_timeout)
    {
        g_source_destroy (g_main_context_find_source_by_id (
            NULL, view->priv->time_timeout));
        view->priv->time_timeout = 0;
        gtk_label_set_text (GTK_LABEL (view->priv->timeLabel), "00:00");
    }

    soup_session_remove_feature_by_type (view->priv->session,
                                         g_type_from_name ("WebKitSoupAuthDialog"));

    soup_session_remove_feature_by_type (view->priv->session,
                                         soup_cookie_jar_get_type ());
    soup_session_add_feature_by_type (view->priv->session,
                                      soup_cookie_jar_get_type ());

    /* set the history limit to 0 to forget the previous user session */
    list = webkit_web_back_forward_list_new_with_web_view (view->webview);
    webkit_web_back_forward_list_set_limit (list, 0);

    view->priv->state = WK_VIEW_STATE_NONE;

    wk_view_show_toolbar (view, FALSE);

    /* manage the "goodbye" page */
    if (view->priv->settings->post_session_uri)
    {
        webkit_web_view_load_uri (view->webview,
                                  view->priv->settings->post_session_uri);
        g_timeout_add_seconds (view->priv->settings->post_session_uri_time,
                               (GSourceFunc) _view_post_session_uri, view);

        return;
    }

    if (view->priv->settings->after_web_session == AFTER_WEB_SESSION_SLIDESHOW)
        wk_view_start_slideshow (view);
    else if (view->priv->settings->after_web_session == AFTER_WEB_SESSION_SESSION)
        wk_view_start_session (view);
    else
        _view_go_home (NULL, view);
}

void
wk_view_add_time (WkView *view, guint seconds)
{
    g_return_if_fail (WK_IS_VIEW (view));

    if (!view->priv->settings->with_timer)
        return;

    gchar *time_str = NULL;

    view->priv->time += seconds;
    g_signal_emit (G_OBJECT (view), fe_signals[TIME_ADDED_SIGNAL], 0);

    seconds_to_string (view->priv->time, &time_str);
    gtk_label_set_text (GTK_LABEL (view->priv->timeLabel), time_str);

    if (view->priv->time == seconds) {
        wk_view_stop_slideshow (view);
        wk_view_start_session (view);
    }
}

void
wk_view_start (WkView *view)
{
    gint status;

    if (view->priv->settings->prestart == NULL)
    {
        if (view->priv->settings->start_with_session)
        {
            if (view->priv->settings->with_timer)
                wk_view_add_time (view, view->priv->settings->start_with_time);
            else
                wk_view_start_session (view);
        }
        else
        {
            wk_view_start_slideshow (view);
        }

        return;
    }

    if (g_spawn_command_line_sync (view->priv->settings->prestart,
                                   NULL, NULL, &status, NULL)
        && WIFEXITED (status)
        && (WEXITSTATUS (status) == 0))
    {
        wk_view_start_session (view); /* FIXME: how do we start the timer? */
    }
    else
    {
        wk_view_start_slideshow (view);
    }
}

void
wk_view_clear_history (WkView *view)
{
    WebKitWebBackForwardList *list;

    soup_session_remove_feature_by_type (view->priv->session,
                                         soup_cookie_jar_get_type ());
    soup_session_add_feature_by_type (view->priv->session,
                                      soup_cookie_jar_get_type ());

    /* set the history limit to 0 to forget the previous user session */
    list = webkit_web_back_forward_list_new_with_web_view (view->webview);
    webkit_web_back_forward_list_set_limit (list, 0);
    webkit_web_back_forward_list_set_limit (list,
                                            view->priv->settings->max_history);

    _view_set_history_buttons (view);
}

void
wk_view_url_entry_grab_focus (WkView *view)
{
    gtk_widget_grab_focus (view->priv->urlEntry);
}

void
wk_view_select_item_nth (WkView *view, gint nth)
{
    if (!view->priv->playlist)
        return;

    if ((nth+1) > g_list_length (view->priv->playlist))
        return;

    _view_kill_slideshow_timeout (view);
    view->priv->slideshow_current = g_list_nth (
        view->priv->playlist, nth);

    _view_show_current_item (view, view->priv->slideshow_timeout);
}

void
wk_view_select_item_prev (WkView *view, gboolean show, gboolean set_timeout)
{
    if (!view->priv->playlist)
        return;

    view->priv->slideshow_current = g_list_previous (
            view->priv->slideshow_current);

    if (!view->priv->slideshow_current)
    {
        view->priv->slideshow_current = g_list_last (
            view->priv->playlist);
    }

    if (show)
    {
        _view_kill_slideshow_timeout (view);
        _view_show_current_item (view, set_timeout);
    }
}

void
wk_view_select_item_next (WkView *view, gboolean show, gboolean set_timeout)
{
    if (!view->priv->playlist)
        return;

    if (!view->priv->slideshow_current)
        view->priv->slideshow_current = g_list_first (view->priv->playlist);
    else
        view->priv->slideshow_current = g_list_next (view->priv->slideshow_current);

    if (!view->priv->slideshow_current)
        view->priv->slideshow_current = g_list_first (view->priv->playlist);

    if (show)
    {
        _view_kill_slideshow_timeout (view);
        _view_show_current_item (view, set_timeout);
    }
}

void
wk_view_get_current_item_position (WkView *view, guint *i)
{
    if (!view->priv->playlist)
        return;

    *i = g_list_position (view->priv->playlist,
                          view->priv->slideshow_current);
}

static void
playlist_append_str (gpointer data, gpointer user_data)
{
    SlideshowItem *item;
    gchar *ret = NULL;
    gchar **txt = (gchar **) user_data;

    item = (SlideshowItem *) data;

    if (*txt == NULL)
        *txt = g_strdup (item->uri);
    else
    {
        ret = g_strdup_printf ("%s\n%s", *txt, item->uri);
        g_free (*txt);
        *txt = ret;
    }
}

void
wk_view_set_playlist (WkView *view,
                      gchar  *playlist)
{
    gchar *filename = NULL;
    gchar *data = NULL;

    if (!g_regex_match_simple ("^[a-z]*://", playlist, G_REGEX_CASELESS, 0))
    {
        g_file_get_contents (playlist, &data, NULL, NULL);
    }
    else if (g_regex_match_simple ("^file://", playlist, G_REGEX_CASELESS, 0))
    {
        if ((filename = g_filename_from_uri (playlist, NULL, NULL)) != NULL)
        {
            g_file_get_contents (filename, &data, NULL, NULL);
            g_free (filename);
        }
    }
    else
    {
        data = get_remote_data (playlist);
    }

    if (!data)
        return;

    wk_settings_free_slideshow (view->priv->settings);
    wk_settings_set_slideshow_from_contents (view->priv->settings, data);
    wk_view_reload_playlist (view);
    view->priv->slideshow_current = g_list_first (view->priv->playlist);
    _view_kill_slideshow_timeout (view);
    _view_show_current_item (view, TRUE);

    g_free (data);
}

void
wk_view_get_playlist_as_str (WkView *view, gchar **playlist)
{
    gchar *data = NULL;

    if (!view->priv->playlist)
    {
        *playlist = NULL;
        return;
    }

    g_list_foreach (view->priv->playlist, playlist_append_str, &data);

    g_free (*playlist);
    *playlist = data;
}

void
wk_view_remove_item (WkView *view, guint index)
{
    SlideshowItem *item;
    guint current_item;

    current_item = g_list_position (view->priv->playlist,
                                    view->priv->slideshow_current);
    item = g_list_nth_data (view->priv->playlist, index);

    if (current_item == index)
    {
        gint  old_length;

        old_length = g_list_length (view->priv->playlist);

        /* If we are going to remove the current item, step to the next one
         * if there is one or stop the slideshow if it was the last one */
        if (old_length > 1)
        {
            _view_kill_slideshow_timeout (view);
            wk_view_select_item_next (view, TRUE, TRUE);
            view->priv->playlist = g_list_remove (view->priv->playlist, item);
        }
        else
        {
            gtk_image_set_black (view->priv->black_img, view);
            gtk_notebook_set_current_page (GTK_NOTEBOOK (view), VIEW_PAGE_BLACK);
            wk_view_stop_slideshow (view);
            view->priv->playlist = g_list_remove (view->priv->playlist, item);
            g_list_free (view->priv->playlist);
            view->priv->playlist = NULL;
            view->priv->slideshow_current = NULL;
        }

        g_free (item);

        return;
    }

    view->priv->playlist = g_list_remove (view->priv->playlist, item);
    g_free (item);
}

void
wk_view_add_item (WkView *view, gint index, gchar *uri, gint time)
{
    SlideshowItem *item;

    g_return_if_fail (uri != NULL);

    if (time < 0)
        time = 5;

    item = g_new0 (SlideshowItem, 1);
    item->uri = g_strdup (uri);
    item->time = time;

    if (index < 0)
        view->priv->playlist = g_list_append (view->priv->playlist, item);
    else
        view->priv->playlist = g_list_insert (view->priv->playlist,
                                              item, index);
}

void
wk_view_reload_playlist (WkView *view)
{
    if (view->priv->playlist)
    {
        g_list_foreach (view->priv->playlist, (GFunc) del_slide_item, NULL);
        g_list_free (view->priv->playlist);
    }

    view->priv->playlist = wk_settings_expand_playlist (view->priv->settings);
}

void
wk_view_pause (WkView *view)
{
    if (view->priv->slideshow_timeout)
        _view_kill_slideshow_timeout (view);
}

void
wk_view_play (WkView *view)
{
    SlideshowItem *item = NULL;

    if (view->priv->state != WK_VIEW_STATE_DISPLAY)
    {
        /* Slideshow is not started, start it */
        wk_view_start_slideshow (view);
        return;
    }

    /* Empty playlist */
    if (!view->priv->playlist)
        return;

    if (view->priv->slideshow_timeout)
    /* Slideshow already running */
        return;

    item = (SlideshowItem *) view->priv->slideshow_current->data;

    _view_display_set_timeout (view, item, TRUE);
}

void
wk_view_is_playing (WkView *view, gboolean *playing)
{
  if (!view->priv->slideshow_timeout)
      *playing = FALSE;
  else
      *playing = TRUE;
}
