/**
 * Copyright (C) 2009-2010 Gauvain Pocentek <gpocentek@linutop.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <glib/gstdio.h>

#include "settings.h"
#include "gtk-extensions.h"

#include "xfce_framebox.h"
#include "config-kiosk.h"

#include "utils.h"

#define LISTURI SYSCONFDIR "/xdg/webkiosk/list"

ConfigItemDesc kiosk_descs[] =
{
    { "main", "can_end_session", CONFIG_ITEM_TYPE_BOOLEAN, "false" },
    { "main", "can_access_fs", CONFIG_ITEM_TYPE_BOOLEAN, "false" },
    { "main", "with_clear", CONFIG_ITEM_TYPE_BOOLEAN, "true" },
    { "main", "default_url", CONFIG_ITEM_TYPE_STRING, "http://linutop.com" },
    { "main", "max_history", CONFIG_ITEM_TYPE_INT, "50" },
    { "main", "max_windows", CONFIG_ITEM_TYPE_INT, "0" },

    { "main", "start_with_session", CONFIG_ITEM_TYPE_BOOLEAN, "true" },
    { "main", "hide_toolbar", CONFIG_ITEM_TYPE_BOOLEAN, "false" },
    { "main", "post_session_uri", CONFIG_ITEM_TYPE_STRING, "" },
    { "main", "post_session_uri_time", CONFIG_ITEM_TYPE_INT, "" },
    /* { "main", "prestart", CONFIG_ITEM_TYPE_STRING, "" }, */

    { "main", "list_mode", CONFIG_ITEM_TYPE_STRING, "none" },
    { "main", "list_use_regex", CONFIG_ITEM_TYPE_BOOLEAN, "false" },
    { "main", "list_uri", CONFIG_ITEM_TYPE_STRING, "list" },

    { "main", "after_web_session", CONFIG_ITEM_TYPE_STRING, "homepage" },

    { NULL }
};

const gchar *combo_list_types[3][2] =
{
    { "none", N_("None") },
    { "whitelist", N_("Whitelist") },
    { "blacklist", N_("Blacklist") },
};

const gchar *combo_after_session[3][2] =
{
    { "homepage", N_("Show the homepage") },
    { "slideshow", N_("Start a slideshow") },
    { "session", N_("Re-start a session") },
};

static GHashTable *hash;
static GtkListStore *list_store, *type_store, *after_store;
static GtkWidget *combo, *after_combo;
static GtkWidget *treeview;
static GtkWidget *add_btn, *rm_btn;
static GtkWidget *advanced_exp;
static WkKeyFile *kfp;

inline GHashTable *
config_kiosk_get_hash (WkKeyFile *kf)
{
    kfp = kf;
    hash = handled_settings_get_from_array (kf, kiosk_descs);
    return hash;
}


static void
populate_stores ()
{
    gchar *listfile;
    ConfigItem *item_file, *item_mode;
    gchar *contents;
    gchar **lines;
    gchar **linep;
    gchar *mode;
    GtkTreeIter iter;
    int i;
    gboolean valid = FALSE;

    /* combo box for ater web session */
    valid = FALSE;
    item_mode = g_hash_table_lookup (hash, "main:after_web_session");
    mode = (gchar *) wk_config_item_get_value (item_mode);
    for (i=0;i<3;i++)
    {
        gtk_list_store_append (after_store, &iter);
        gtk_list_store_set (after_store, &iter, 0, combo_after_session[i][0],
                                                1, _(combo_after_session[i][1]),
                                                -1);
        if (strcmp (mode, combo_after_session[i][0]) == 0)
        {
            gtk_combo_box_set_active (GTK_COMBO_BOX (after_combo), i);
            valid = TRUE;
        }
    }
    if (! valid)
        gtk_combo_box_set_active (GTK_COMBO_BOX (after_combo), 0);

    /* combo box for list type */
    item_mode = g_hash_table_lookup (hash, "main:list_mode");
    mode = (gchar *) wk_config_item_get_value (item_mode);
    for (i=0;i<3;i++)
    {
        gtk_list_store_append (type_store, &iter);
        gtk_list_store_set (type_store, &iter, 0, combo_list_types[i][0],
                                               1, _(combo_list_types[i][1]),
                                               -1);
        if (strcmp (mode, combo_list_types[i][0]) == 0)
        {
            gtk_combo_box_set_active (GTK_COMBO_BOX (combo), i);
            valid = TRUE;
        }
    }
    if (! valid)
        gtk_combo_box_set_active (GTK_COMBO_BOX (combo), 0);

    /* list itself */
    item_file = g_hash_table_lookup (hash, "main:list_uri");

    if (strcmp ((gchar *) wk_config_item_get_value (item_file), "") == 0)
    {
        listfile = wk_key_file_build_filename (kfp, LISTURI);
        wk_config_item_set_value (
            item_file, (gpointer) g_strdup (listfile), TRUE);
    }
    else
        listfile = wk_key_file_build_filename (kfp,
                            wk_config_item_get_value (item_file));

    if (! g_file_get_contents (listfile, &contents, NULL, NULL))
        return;

    lines = g_strsplit (contents, "\n", -1);
    for (linep = lines; *linep; linep++)
    {
        if (strlen (*linep) == 0)
            continue;

        gtk_list_store_append (list_store, &iter);
        gtk_list_store_set (list_store, &iter, 0, *linep, -1);
    }
    g_strfreev (lines);
    g_free (listfile);
    g_free (contents);
}

static void
on_treeview_changed (GtkTreeView *treeview, gpointer data)
{
    GtkTreeIter iter;
    GtkTreeModel *model = GTK_TREE_MODEL (list_store);

    if (gtk_tree_selection_get_selected (
            gtk_tree_view_get_selection (treeview), &model, &iter))
        gtk_widget_set_sensitive (rm_btn, TRUE);
    else
        gtk_widget_set_sensitive (rm_btn, FALSE);
}

static void
on_add_btn_clicked (GtkButton *btn, gpointer data)
{
    GtkWidget *d, *add_entry;
    const gchar *text;
    GtkTreeIter iter;

    d = gtk_dialog_new_with_buttons (_("Add a URL"),
                                     NULL,
                                     GTK_DIALOG_MODAL | GTK_DIALOG_NO_SEPARATOR,
                                     GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                     GTK_STOCK_ADD, GTK_RESPONSE_OK,
                                     NULL);
    add_entry = gtk_entry_new ();
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (d)->vbox), add_entry, TRUE, TRUE, 10);
    gtk_widget_show (add_entry);

    if (gtk_dialog_run (GTK_DIALOG (d)) != GTK_RESPONSE_OK)
        goto out;

    text = gtk_entry_get_text (GTK_ENTRY (add_entry));
    if (strlen (text) < 1)
        goto out;

    gtk_list_store_append (list_store, &iter);
    gtk_list_store_set (list_store, &iter, 0, g_strdup (text), -1);

out:
    gtk_widget_destroy (d);
    return;
}

static void
on_rm_btn_clicked (GtkButton *btn, gpointer data)
{
    GtkTreeIter iter;
    GtkTreeModel *model = GTK_TREE_MODEL (list_store);

    if (!gtk_tree_selection_get_selected (
        gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview)), &model, &iter))
        return;

    gtk_list_store_remove (list_store, &iter);

    on_treeview_changed (GTK_TREE_VIEW (treeview), NULL);
}

static void
on_after_combo_changed (GtkComboBox *combo, ConfigItem *item)
{
    GtkTreeIter iter;
    gchar *what = NULL;
    gboolean b;

    gtk_combo_box_get_active_iter (combo, &iter);
    gtk_tree_model_get (GTK_TREE_MODEL (after_store), &iter, 0, &what, -1);

    /* 'what' will be freed with the ConfigItem */
    wk_config_item_set_value (item, what, TRUE);

    b = (g_strcmp0 (what, "slideshow") == 0);
    gtk_widget_set_sensitive (display_widget, b);
    gtk_widget_set_sensitive (remote_widget, b);
}

static void
on_combo_changed (GtkComboBox *combo, ConfigItem *item)
{
    GtkTreeIter iter;
    gchar *mode = NULL;

    gtk_combo_box_get_active_iter (combo, &iter);
    gtk_tree_model_get (GTK_TREE_MODEL (type_store), &iter, 0, &mode, -1);

    /* mode will be freed with the ConfigItem */
    wk_config_item_set_value (item, mode, TRUE);
}

GtkWidget *
config_kiosk_get_widget (GHashTable *hash)
{
    GtkWidget *mvbox; /* returned widget */
    GtkWidget *label, *frame, *box, *box2, *box3, *hbox, *cvbox;
    GtkWidget *scw;
    GtkSizeGroup *size_group;
    GtkCellRenderer *renderer;
    GtkTreeViewColumn *column;

    mvbox = gtk_vbox_new (FALSE, 2);

    hbox = gtk_hbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX (mvbox), hbox, FALSE, FALSE, 0);

    cvbox = gtk_vbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX (hbox), cvbox, FALSE, FALSE, 0);

    frame = xfce_framebox_new (_("Interface"), TRUE);
    box = gtk_vbox_new (FALSE, 2);
    xfce_framebox_add (XFCE_FRAMEBOX (frame), box);
    gtk_box_pack_start (GTK_BOX (cvbox), frame, FALSE, FALSE, 6);

    gtk_box_pack_start (GTK_BOX (box),
                        wk_gtk_check_button_new (_("Hide the tool bar"),
                            g_hash_table_lookup (hash, "main:hide_toolbar"),
                            FALSE),
                        FALSE, FALSE, 0);

    gtk_box_pack_start (GTK_BOX (box),
                        wk_gtk_check_button_new (_("User can end his web session"),
                            g_hash_table_lookup (hash, "main:can_end_session"),
                            FALSE),
                        FALSE, FALSE, 0);

    gtk_box_pack_start (GTK_BOX (box),
                        wk_gtk_check_button_new (_("Display the 'clear' button"),
                            g_hash_table_lookup (hash, "main:with_clear"),
                            FALSE),
                        FALSE, FALSE, 0);

    gtk_box_pack_start (GTK_BOX (box),
                        wk_gtk_check_button_new (_("Begin a web session at startup"),
                            g_hash_table_lookup (hash, "main:start_with_session"),
                            FALSE),
                        FALSE, FALSE, 0);

    box2 = gtk_hbox_new (FALSE, 2);
    label = gtk_label_new (_("Home page: "));
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
    gtk_box_pack_start (GTK_BOX (box2), label, FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (box2),
                        wk_gtk_entry_new (
                            g_hash_table_lookup (hash, "main:default_url")),
                        FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (box), box2, FALSE, FALSE, 0);

    /* restrictions */
    cvbox = gtk_vbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX (hbox), cvbox, TRUE, TRUE, 0);

    frame = xfce_framebox_new (_("Restrictions"), TRUE);
    box = gtk_vbox_new (FALSE, 2);
    xfce_framebox_add (XFCE_FRAMEBOX (frame), box);
    gtk_box_pack_start (GTK_BOX (cvbox), frame, FALSE, FALSE, 6);

    type_store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING);
    list_store = gtk_list_store_new (1, G_TYPE_STRING);
    after_store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING);

    combo = gtk_combo_box_new_with_model (GTK_TREE_MODEL (type_store));
    renderer = gtk_cell_renderer_text_new ();
    gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (combo), renderer, TRUE);
    gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (combo), renderer, "text", 1);
    g_signal_connect (G_OBJECT (combo), "changed",
                      G_CALLBACK (on_combo_changed),
                      g_hash_table_lookup (hash, "main:list_mode"));

    box2 = gtk_hbox_new (FALSE, 6);

    scw = gtk_scrolled_window_new (NULL, NULL);
    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scw),
                                    GTK_POLICY_AUTOMATIC,
                                    GTK_POLICY_AUTOMATIC);

    treeview = gtk_tree_view_new_with_model (GTK_TREE_MODEL (list_store));
    gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (treeview), FALSE);
    renderer = gtk_cell_renderer_text_new ();
    column = gtk_tree_view_column_new_with_attributes (
        "",
        renderer,
        "text", 0,
        NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);

    box3 = gtk_vbox_new (FALSE, 2);

    add_btn = gtk_button_new_from_stock (GTK_STOCK_ADD);
    rm_btn = gtk_button_new_from_stock (GTK_STOCK_DELETE);
    gtk_widget_set_sensitive (rm_btn, FALSE);

    gtk_box_pack_start (GTK_BOX (box), combo, FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (box),
                        wk_gtk_check_button_new (_("Use regular expressions"),
                            g_hash_table_lookup (hash, "main:list_use_regex"),
                            FALSE),
                        FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (box), box2, TRUE, TRUE, 2);
    gtk_box_pack_start (GTK_BOX (box2), scw, TRUE, TRUE, 2);
    gtk_container_add (GTK_CONTAINER (scw), treeview);
    gtk_box_pack_start (GTK_BOX (box2), box3, FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (box3), add_btn, FALSE, FALSE, 0);
    gtk_box_pack_start (GTK_BOX (box3), rm_btn, FALSE, FALSE, 0);

    gtk_box_pack_start (GTK_BOX (box),
                        wk_gtk_check_button_new (_("User can browse the file system"),
                            g_hash_table_lookup (hash, "main:can_access_fs"),
                            FALSE),
                        FALSE, FALSE, 0);

    g_signal_connect (G_OBJECT (treeview), "cursor-changed",
                      G_CALLBACK (on_treeview_changed), NULL);
    g_signal_connect (G_OBJECT (add_btn), "clicked",
                      G_CALLBACK (on_add_btn_clicked), NULL);
    g_signal_connect (G_OBJECT (rm_btn), "clicked",
                      G_CALLBACK (on_rm_btn_clicked), NULL);

    /* advanced settings */
    advanced_exp = gtk_expander_new (_("<b>Advanced settings</b>"));
    gtk_expander_set_use_markup (GTK_EXPANDER (advanced_exp), TRUE);
    box = gtk_vbox_new (FALSE, 2);
    gtk_box_pack_start (GTK_BOX (mvbox), advanced_exp, FALSE, FALSE, 0);
    frame = xfce_framebox_new (NULL, TRUE);
    xfce_framebox_add (XFCE_FRAMEBOX (frame), box);
    gtk_container_add (GTK_CONTAINER (advanced_exp), frame);

    size_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);

    box2 = gtk_hbox_new (FALSE, 2);
    label = gtk_label_new (_("Max items in history: "));
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
    gtk_size_group_add_widget (size_group, label);
    gtk_box_pack_start (GTK_BOX (box2), label, FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (box2),
                        wk_gtk_spin_button_new_with_range (0, 100, 1,
                            g_hash_table_lookup (hash, "main:max_history")),
                        FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (box), box2, FALSE, FALSE, 2);

    box2 = gtk_hbox_new (FALSE, 2);
    label = gtk_label_new (_("Max opened windows: "));
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
    gtk_size_group_add_widget (size_group, label);
    gtk_box_pack_start (GTK_BOX (box2), label, FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (box2),
                        wk_gtk_spin_button_new_with_range (0, 50, 1,
                            g_hash_table_lookup (hash, "main:max_windows")),
                        FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (box), box2, FALSE, FALSE, 2);

    box2 = gtk_hbox_new (FALSE, 2);
    label = gtk_label_new (_("Post session URL: "));
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
    gtk_size_group_add_widget (size_group, label);
    gtk_box_pack_start (GTK_BOX (box2), label, FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (box2),
                        wk_gtk_entry_new (
                            g_hash_table_lookup (hash, "main:post_session_uri")),
                        FALSE, FALSE, 2);
    label = gtk_label_new (_(" shown "));
    gtk_box_pack_start (GTK_BOX (box2), label, FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (box2),
                        wk_gtk_spin_button_new_with_range (0, 100, 1,
                            g_hash_table_lookup (hash, "main:post_session_uri_time")),
                        FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (box2), gtk_label_new (_("second(s)")),
                        FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (box), box2, FALSE, FALSE, 2);

    box2 = gtk_hbox_new (FALSE, 2);
    label = gtk_label_new (_("When a web session ends: "));
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
    gtk_size_group_add_widget (size_group, label);
    gtk_box_pack_start (GTK_BOX (box2), label, FALSE, FALSE, 2);
    after_combo = gtk_combo_box_new_with_model (GTK_TREE_MODEL (after_store));
    renderer = gtk_cell_renderer_text_new ();
    gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (after_combo), renderer, TRUE);
    gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (after_combo), renderer, "text", 1);
    gtk_box_pack_start (GTK_BOX (box2), after_combo, FALSE, FALSE, 2);

    gtk_box_pack_start (GTK_BOX (box), box2, FALSE, FALSE, 2);

    populate_stores ();

    g_signal_connect (G_OBJECT (after_combo), "changed",
                      G_CALLBACK (on_after_combo_changed),
                      g_hash_table_lookup (hash, "main:after_web_session"));

    return mvbox;
}

static gboolean
append_data (GtkTreeModel *model, GtkTreePath *path,
             GtkTreeIter *iter, gpointer data)
{
    gchar *line = NULL;
    gchar *tmp = NULL;
    gchar **datap = (gchar **) data;

    gtk_tree_model_get (model, iter, 0, &line, -1);

    if (*datap == NULL)
        *datap = g_strdup (line);
    else
    {
        tmp = g_strconcat (*datap, "\n", line, NULL);
        g_free (*datap);
        *datap = tmp;
    }

    g_free (line);

    return FALSE;
}

void
config_kiosk_save_config (WkKeyFile *kf, GHashTable *hash)
{
    ConfigItem *item;
    gchar *filename;
    gchar *tmpname = NULL;
    gchar *data = NULL;

    handled_settings_set_from_hash (kf, hash);

    item = g_hash_table_lookup (hash, "main:list_uri");
    filename = wk_key_file_build_filename (kf, wk_config_item_get_value (item));
    tmpname = g_strdup_printf ("%s.tmp", filename);

    gtk_tree_model_foreach (GTK_TREE_MODEL (list_store), append_data, &data);

    if (!data)
        data = g_strdup ("");

    if (!create_dir_for_file (tmpname)
        || !g_file_set_contents (tmpname, data, -1, NULL)
        || (g_rename (tmpname, filename) != 0))
    {
        gtk_dialog_run (GTK_DIALOG (gtk_message_dialog_new (
            NULL, 0, GTK_MESSAGE_ERROR,
            GTK_BUTTONS_CLOSE,
            _("Unable to save the listing file "
            "to\n%s"), filename)));
    }

    g_free (filename);
    g_free (tmpname);
    g_free (data);
}
